#!/usr/bin/env python
# Copyright (c) 2015 Sergey Bunatyan <sergey.bunatyan@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


class Singleton(object):
    """
    For example:

    >>> class ConcreteSingleton(Singleton):
    ...     pass
    """

    _instance = None

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = (super(Singleton, cls)
                             .__new__(cls, *args, **kwargs))
        return cls._instance


def get_fqcn(obj):
    """Get %(module_path)s.%(class_name)s of an object.

    FQCN stands for Fully Qualified Class Name.
    """
    return "%(module_path)s.%(class_name)s" % dict(
        module_path=obj.__class__.__module__,
        class_name=obj.__class__.__name__)
